unit AppointmentView;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DsgnIntf, TimePeriodList, Menus, Grids;
{**************************************************************************************************}
type
TFrmPropAppointListDlg = class(TForm)
   OKBtn: TButton;
   CancelBtn: TButton;
    BtnNew: TBitBtn;
    BtnEdit: TBitBtn;
    BtnDelete: TBitBtn;
    PopupMenu: TPopupMenu;
    Edit1: TMenuItem;
    Edit2: TMenuItem;
    Delete1: TMenuItem;
    AppGrid: TStringGrid;
    procedure BtnNewClick(Sender: TObject);
    procedure BtnEditClick(Sender: TObject);
    procedure BtnDeleteClick(Sender: TObject);
    procedure Edit1Click(Sender: TObject);
    procedure Edit2Click(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure AppGridDblClick(Sender: TObject);
private
  m_TAppointments: TTimePeriodList;

  function  Init(aList: TTimePeriodList): Boolean;
  procedure Deinit;

  procedure AddToLB;
  procedure ClearLB;
  procedure UpdateLB;

public
   function Execute(aList: TTimePeriodList): Boolean;

   // Commands
   procedure DoNew   ;
   procedure DoEdit  ;
   procedure DoDelete;
end;
{**************************************************************************************************}
var
  FrmPropAppointListDlg: TFrmPropAppointListDlg;
{**************************************************************************************************}
implementation
Uses Dialogs            ,
     TimePeriod         ,
     Plan               ,
     FrmAppointmentIODialog,
     Utils;

{$R *.DFM}
{**************************************************************************************************}
function TFrmPropAppointListDlg.Execute(aList: TTimePeriodList): Boolean;
begin
   Result := FALSE;
   if(Init(aList)) then
      Result := (ShowModal = mrOk);

   Deinit;
end;
{**************************************************************************************************}
function TFrmPropAppointListDlg.Init(aList: TTimePeriodList): Boolean;
begin
   try
      m_TAppointments := NIL;
      Result := Assigned(aList);
      if(Result) then begin
         m_TAppointments := aList;
         AddToLB;
      end;
   except
      on E:exception do begin
         Result := FALSE;
         MessageDlg('Error while initilizing property-dialog.'+#13+#10+E.Message, mtInformation, [mbOk] ,0);
      end;
   end;
end;
{**************************************************************************************************}         
procedure TFrmPropAppointListDlg.Deinit;
begin
   ClearLB;
   m_TAppointments := NIL;
end;
{**************************************************************************************************}         
procedure TFrmPropAppointListDlg.AddToLB;
var
  aItem     : TTimePeriod;
  nCounter  : integer    ;
begin
   AppGrid.Cells[0,0] := 'Start:';
   AppGrid.Cells[1,0] := 'End:';
   AppGrid.Cells[2,0] := 'Display string:';
   AppGrid.RowCount := lnMax(m_TAppointments.Count+1 ,2);

   for nCounter := 0 to m_TAppointments.Count - 1 do begin
      aItem := m_TAppointments.Items[nCounter];
      if(Assigned(aItem)) then begin
         AppGrid.Cells[0,nCounter+1] := DateTimeToStr(aItem.StartDate);
         AppGrid.Cells[1,nCounter+1] := DateTimeToStr(aItem.EndDate);
         AppGrid.Cells[2,nCounter+1] := aItem.DisplayString
      end;
   end;
end;
{**************************************************************************************************}         
procedure TFrmPropAppointListDlg.ClearLB;
var
  nCounter: integer;
begin
   for nCounter := 1 to AppGrid.RowCount do 
      AppGrid.Rows[nCounter].Clear;
end;
{**************************************************************************************************}         
procedure TFrmPropAppointListDlg.UpdateLB;
begin
   ClearLB;
   AddToLB;
end;
{**************************************************************************************************}         
{**************************************************************************************************}         
// Commands
{**************************************************************************************************}         
procedure TFrmPropAppointListDlg.DoNew   ;
var
  aItem: TTimePeriod;
begin
   try
      aItem := TTimePeriod.CreateWithParam(now, now+1, NIL, 'NEW');
      FrmAppointmentIODlg := TFrmAppointmentIODlg.Create(Self);
      if(FrmAppointmentIODlg.Execute(aItem)) then begin
         try
            m_TAppointments.Add(aItem);
            UpdateLB;
         except
            aItem.Free;
            Raise;
         end;
      end
      else
         aItem.Free;
   except
      on E:Exception do 
         MessageDlg('Error while adding an new appointment.'+#13+#10+E.Message, mtInformation, [mbOk] , 0);
   end;
end;
{**************************************************************************************************}         
procedure TFrmPropAppointListDlg.DoEdit  ;
var
  aItem   : TTimePeriod;
  aTmpItem: TTimePeriod;
begin
   try
      if(m_TAppointments.Count =0) then begin
         MessageDlg('No appointment''s.', mtInformation, [mbOk] ,0);
         DoNew;
         Exit;
      end;
      aItem     := m_TAppointments.Items[AppGrid.Row-1];
      aTmpItem  := TTimePeriod.CreateWithObj(aItem);
      m_TAppointments.Delete(m_TAppointments.IndexOf(aItem));

      aItem := TTimePeriod.CreateWithObj(aTmpItem);
      FrmAppointmentIODlg := TFrmAppointmentIODlg.Create(NIL);
      try
         if(FrmAppointmentIODlg.Execute(aItem)) then begin
             if(Not m_TAppointments.IsOverlapping(aItem)) then begin
                m_TAppointments.Add(aItem);
                aTmpItem.Free;   
             end
             else begin
                m_TAppointments.Add(aTmpItem);
                aItem.Free;
                MessageDlg('The appointment overlapps an other.', mtError, [mbOk] ,0);
             end;
         end
         else begin
            m_TAppointments.Add(aTmpItem);
            aItem.Free;
         end;

      finally
         FrmAppointmentIODlg.Free;
      end;
   except
      on E:Exception do 
         MessageDlg('Error while adding an new appointment.'+#13+#10+E.Message, mtInformation, [mbOk] , 0);
   end;
   UpdateLB;
end;
{**************************************************************************************************}         
procedure TFrmPropAppointListDlg.DoDelete;
begin
   if(m_TAppointments.Count =0) then begin
      MessageDlg('No appointment''s.', mtInformation, [mbOk] ,0);
      Exit;
   end;

   m_TAppointments.Delete(AppGrid.Row-1);
   UpdateLB;
end;
{**************************************************************************************************}         
{**************************************************************************************************}
{* Dialog fr Property - Editor fr AppointmentList (Klasse in TPlan)(ENDE)                *}
{**************************************************************************************************}
{**************************************************************************************************}
{* Eigene Funktionen (ENDE)                                                                       *}
{**************************************************************************************************}
procedure TFrmPropAppointListDlg.BtnNewClick(Sender: TObject);
begin
   DoNew;
end;
{**************************************************************************************************}
procedure TFrmPropAppointListDlg.BtnEditClick(Sender: TObject);
begin
   DoEdit;
end;
{**************************************************************************************************}
procedure TFrmPropAppointListDlg.BtnDeleteClick(Sender: TObject);
begin
   DoDelete;
end;
{**************************************************************************************************}
procedure TFrmPropAppointListDlg.Edit1Click(Sender: TObject);
begin
   DoNew;
end;
{**************************************************************************************************}
procedure TFrmPropAppointListDlg.Edit2Click(Sender: TObject);
begin
   DoEdit;
end;
{**************************************************************************************************}
procedure TFrmPropAppointListDlg.Delete1Click(Sender: TObject);
begin
  DoDelete;
end;
{**************************************************************************************************}
procedure TFrmPropAppointListDlg.AppGridDblClick(Sender: TObject);
begin
   DoEdit;
end;
{**************************************************************************************************}
end.
